#!/usr/bin/env python
"""
:Author Patrik Valkovic
:Created 01.08.2017 08:54
:Licence GNUv3
Part of grammpy

"""

from .GrammpyException import GrammpyException

class CannotConvertException(GrammpyException):
    pass